# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2013 Canonical
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import mox
import os

from nova.compute import flavors
from nova.compute import power_state
from nova import context
from nova import db
from nova.network import model as network_model
from nova import test
from nova.tests import fake_network_cache_model
from nova import utils
from nova.virt import images
from nova.virt.lxc import driver as lxc_connection
from nova.virt.lxc import vmutils


class LXCDriverTestCase(test.TestCase):
    def setUp(self):
        super(LXCDriverTestCase, self).setUp()
        self.flags(instances_path=self.useFixture(fixtures.TempDir()).path)
        self.lxc_connection = lxc_connection.LXCDriver(None)

        instance_type = db.flavor_get(context.get_admin_context(), 5)
        sys_meta = flavors.save_flavor_info({}, instance_type)

        self.instance = {
            'uuid': '32dfcb37-5af1-552b-357c-be8c3aa38310',
            'memory_kb': '1024000',
            'basepath': '/some/path',
            'bridge_name': 'br100',
            'vcpus': 2,
            'project_id': 'fake',
            'bridge': 'br101',
            'image_ref': '155d900f-4e14-4e4c-a73d-069cbf4541e6',
            'root_gb': 10,
            'ephemeral_gb': 20,
            'instance_type_id': '5',  # m1.small
            'extra_specs': {},
            'system_metadata': sys_meta}

    def test_get_info_containers_running(self):
        self.mox.StubOutWithMock(vmutils.VMUtils, 'container_exists')
        vmutils.VMUtils.container_exists(mox.IgnoreArg()).AndReturn(True)
        self.mox.ReplayAll()
        state = self.lxc_connection.get_info(self.instance)
        self.assertEqual(state['state'], power_state.RUNNING)

    def test_get_info_containers_not_running(self):
        self.mox.StubOutWithMock(vmutils.VMUtils, 'container_exists')
        vmutils.VMUtils.container_exists(mox.IgnoreArg()).AndReturn(False)
        self.mox.ReplayAll()
        state = self.lxc_connection.get_info(self.instance)
        self.assertEqual(state['state'], power_state.SHUTDOWN)

    def test_get_info_containers_no_status(self):
        self.mox.StubOutWithMock(vmutils.VMUtils, 'container_exists')
        vmutils.VMUtils.container_exists(mox.IgnoreArg()).AndReturn('')
        self.mox.ReplayAll()
        state = self.lxc_connection.get_info(self.instance)
        self.assertEqual(state['state'], power_state.NOSTATE)

    def test_spawn(self):
        def fake_image_fetch(context, image_id, file_path,
                             user_id, project_id):
            pass

        self.stubs.Set(images, 'fetch', fake_image_fetch)
        image_meta = {}
        image_meta['id'] = '666'
        fake_net_info = network_model.NetworkInfo([
            fake_network_cache_model.new_vif()])

        instance_ref = self.instance
        instance_ref['image_ref'] = 123456  # we send an int to test sha1 call
        instance_type = db.flavor_get(context.get_admin_context(),
                                      instance_ref['instance_type_id'])
        sys_meta = flavors.save_flavor_info({}, instance_type)
        instance_ref['system_metadata'] = sys_meta
        instance = db.instance_create(
            context.get_admin_context(), instance_ref)

        self.lxc_connection.spawn(context.get_admin_context(),
                                  self.instance, image_meta, [], '',
                                  fake_net_info)

    def test_get_console(self):
        with utils.tempdir() as tmpdir:
            self.flags(instances_path=tmpdir)

            instance_ref = self.instance
            instance_ref['image_ref'] = 123456
            instance = db.instance_create(
                context.get_admin_context(), instance_ref)

            console_dir = os.path.join(tmpdir, self.instance['uuid'])
            console_log = "%s/console.log" % console_dir
            os.makedirs(console_dir)

            f = open(console_log, 'w')
            f.write('0123456789')
            f.close()

            output = self.lxc_connection.get_console_output(self.instance)

        self.assertEquals('0123456789', output)

    def test_get_host_stats(self):
        host_stats = self.lxc_connection.get_host_stats()
        self.assertIsNotNone(host_stats)
