# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright (c) 2013 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import jinja2
from oslo.config import cfg

from nova.openstack.common.gettextutils import _
from nova.openstack.common import log as logging
from nova import utils
from nova.virt.disk import api as disk
from nova.virt import images
from nova.virt.libvirt import utils as libvirt_utils
from nova.virt.lxc import vmutils

LOG = logging.getLogger(__name__)

CONF = cfg.CONF

MAX_CONSOLE_BYTES = 102400


class VMOps(object):
    def __init__(self):
        self.vmutils = vmutils.VMUtils()

    def spawn(self, context, instance, image_meta, injected_files,
              admin_password, network_info=None, block_device_info=None):
        LOG.info(_('Spawning instance'))
        try:
            instance_path = os.path.join(CONF.instances_path,
                    instance['uuid'])
            image_path = os.path.join(instance_path, 'disk')
            container_rootfs = os.path.join(instance_path, 'rootfs')
            lxc_container = os.path.join(CONF.lxc_path, instance['uuid'])
            console_file = os.path.join(instance_path, 'console.log')

            def _lxc_setup_directories():
                LOG.info(_('Creating initial LXC directories'),
                         instance=instance)
                if not os.path.exists(lxc_container):
                    utils.execute('mkdir', '-p', lxc_container,
                            run_as_root=True)

                if not os.path.exists(container_rootfs):
                    utils.execute('mkdir', '-p', container_rootfs,
                            run_as_root=True)

                if not os.path.exists(instance_path):
                    os.mkdir(instance_path)

            _lxc_setup_directories()

            def _lxc_build_config():
                LOG.info(_('Build LXC configuration file'))
                for vif in network_info:
                    mac_address = vif['address']
                network_bridge = network_info[0]['network']['bridge']

                tmpl_path, tmpl_file = os.path.split(CONF.lxc_config_template)
                env = jinja2.Environment(
                        loader=jinja2.FileSystemLoader(tmpl_path))
                template = env.get_template(tmpl_file)
                lxc_config = template.render(
                                {'container_name': instance['uuid'],
                                 'container_rootfs': container_rootfs,
                                 'container_link': network_bridge,
                                 'container_hwaddr': mac_address,
                                 'console_file': console_file,
                                 'container_fstab': '%s/%s/fstab'
                                     % (CONF.lxc_path, instance['uuid'])})

                config_file = os.path.join(CONF.lxc_path, instance['uuid'],
                                          'config')
                vmutils.write_to_file(config_file, lxc_config)

                fstab_file = os.path.join(CONF.lxc_path, instance['uuid'],
                                         'fstab')
                fstab_config = open(CONF.lxc_fstab_template, 'r').read()
                vmutils.write_to_file(fstab_file, fstab_config)

            _lxc_build_config()

            def _lxc_ready_container():
                LOG.info(_('Downloading and Mounting image'))
                images.fetch(context, instance['image_ref'], image_path,
                             instance['user_id'], instance['project_id'])
                disk.setup_container(image_path,
                                     container_dir=container_rootfs,
                                     use_cow=CONF.use_cow_images)

            _lxc_ready_container()

            def _lxc_start_container():
                LOG.info(_('Starting LXC container'))
                self.vmutils.start_container(instance)

            _lxc_start_container()
        except Exception as ex:
            LOG.exception(ex)

    def destroy(self, instance, network_info, block_device_info=None,
                destroy_disks=True, context=None):
        LOG.info(_('Destroying LXC container'))
        container_dir = os.path.join(CONF.instances_path, instance['uuid'],
                                     'rootfs')
        instance_dir = os.path.join(CONF.instances_path, instance['uuid'])
        disk.teardown_container(container_dir=container_dir)
        self.vmutils.destroy_container(instance)

    def power_off(self, instance):
        self.vmutils.stop_container(instance)

    def power_on(self, instance):
        self.vmutils.start_container(instance)

    def get_console_output(self, instance):
        console_file = os.path.join(CONF.instances_path, instance['uuid'],
                                    'console.log')
        with libvirt_utils.file_open(console_file, 'r') as fp:
            log_data, remaining = utils.last_bytes(fp,
                                                   MAX_CONSOLE_BYTES)
            return log_data

    def list_instances(self):
        return self.vmutils.list_containers()

    def container_exists(self, instance):
        return self.vmutils.container_exists(instance)
